r"""Doctests in TestCase classes

"""
import doctest
import inspect
import os
import re
import sys
import types


__all__ = [
    'doctestmethod',
    'method',

    'docteststring',
    'string',

    'doctestfile',
    'file',

    'doctestfiles',
    'files',
]

_parser = doctest.DocTestParser()


def _testify(name):
    if not name.startswith('test'):
        name = 'test_' + name
    return name


def doctestmethod(test=None, optionflags=0, checker=None):
    """Define a doctest from a method within a unittest.TestCase.

    The method's doc string provides the test source. Its body is
    called before the test and may perform test-specific setup.

    You can pass doctest option flags and a custon checker.

    Variables defined in the enclosing module are available in the test.

    If a test case defines a globs attribute, it must be a dictionary
    and its contents are added to the test globals.

    The test object is available as the variable ``self`` in the test.
    """
    if test is None:
        return lambda test: _doctestmethod(test, optionflags, checker)

    return _doctestmethod(test, optionflags, checker)


#: Alias of `doctestmethod`
method = doctestmethod


def _doctestmethod(test, optionflags, checker):
    doc = test.__doc__
    if not doc:
        raise ValueError(test, "has no docstring")
    setup = test
    name = test.__name__
    path = inspect.getsourcefile(test)
    lineno = inspect.getsourcelines(test)[1]

    fglobs = sys._getframe(3).f_globals

    def test_method(self):
        setup(self)
        _run_test(self, doc, fglobs.copy(), name, path,
                  optionflags, checker, lineno=lineno)

    test_method.__name__ = _testify(name)

    return test_method


def docteststring(test, optionflags=0, checker=None, name=None):
    """Define a doctest from a string within a unittest.TestCase.

    You can pass doctest option flags and a custon checker.

    Variables defined in the enclosing module are available in the test.

    If a test case defines a globs attribute, it must be a dictionary
    and its contents are added to the test globals.

    The test object is available as the variable ``self`` in the test.
    """
    fglobs = sys._getframe(2).f_globals

    def test_string(self):
        _run_test(self, test, fglobs.copy(), '<string>', '<string>',
                  optionflags, checker)
    if name:
        test_string.__name__ = _testify(name)

    return test_string


#: Alias of `docteststring`
string = docteststring

_not_word = re.compile(r'\W')


def doctestfile(path, optionflags=0, checker=None):
    """Define a doctest from a test file within a unittest.TestCase.

    The file path may be relative or absolute. If its relative (the
    common case), it will be interpreted relative to the directory
    containing the referencing module.

    You can pass doctest option flags and a custon checker.

    If a test case defines a globs attribute, it must be a dictionary
    and its contents are added to the test globals.

    The test object is available as the variable ``test`` in the test.

    The resulting object can be used as a function decorator. The
    decorated method is called before the test and may perform
    test-specific setup. (The decorated method's doc string is ignored.)
    """
    base = os.path.dirname(os.path.abspath(
        sys._getframe(2).f_globals['__file__']
        ))
    path = os.path.join(base, path)
    with open(path) as f:
        test = f.read()
    name = os.path.basename(path)

    def test_file(self):
        if isinstance(self, types.FunctionType):
            setup = self

            def test_file_w_setup(self):
                setup(self)
                _run_test(self, test, {}, name, path, optionflags, checker,
                          'test')

            test_file_w_setup.__name__ = _testify(setup.__name__)
            test_file_w_setup.filepath = path
            test_file_w_setup.filename = os.path.basename(path)
            return test_file_w_setup

        _run_test(self, test, {}, name, path, optionflags, checker, 'test')

    test_file.__name__ = name_from_path(path)
    test_file.filepath = path
    test_file.filename = os.path.basename(path)

    return test_file


#: Alias of `doctestfile`
file = doctestfile


def doctestfiles(*paths, **kw):
    """Define doctests from test files in a decorated class.

    Multiple files can be specified. A member is added to the
    decorated class for each file.

    The file paths may be relative or absolute. If relative (the
    common case), they will be interpreted relative to the directory
    containing the referencing module.

    You can pass doctest option flags and a custon checker.

    If a test case defines a globs attribute, it must be a dictionary
    and its contents are added to the test globals.

    The test object is available as the variable ``test`` in the test.

    The resulting object must be used as a class decorator.
    """
    def doctestfiles_(class_):
        for path in paths:
            name = name_from_path(path)
            test = doctestfile(path, **kw)
            test.__name__ = name
            setattr(class_, name, test)

        return class_

    return doctestfiles_


#: Alias of `doctestfiles`
files = doctestfiles


def name_from_path(path):
    return _testify(
        _not_word.sub('_', os.path.splitext(os.path.basename(path))[0])
        )


def _run_test(self, test, globs, name, path,
              optionflags, checker, testname='self', lineno=0):
    globs.update(getattr(self, 'globs', ()))
    globs[testname] = self
    optionflags |= doctest.IGNORE_EXCEPTION_DETAIL
    doctest.DocTestCase(
        _parser.get_doctest(test, globs, name, path, lineno),
        optionflags=optionflags,
        checker=checker,
        ).runTest()



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Exceptions for zope.testing
"""

import warnings


# Tell people to use the builtin module instead.
warnings.warn(
    'zope.testing.exceptions is deprecated in favour of '
    'zope.testrunner.exceptions', DeprecationWarning,
    stacklevel=2)


class DocTestFailureException(AssertionError):
    """Use custom exception for doctest unit test failures"""



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Stack-based test `doctest` ``setUp`` and `tearDown`

See :doc:`../setupstack`
"""

import os
import stat
import tempfile
import unittest


key = '__' + __name__


def globs(test):
    """
    Return the globals for *test*, which can be a `doctest`
    or a regular `unittest.TestCase`.
    """
    try:
        return test.globs
    except AttributeError:
        return test.__dict__


def register(test, function, *args, **kw):
    """
    Add a clean up *function* to be called with
    *args* and *kw* when *test* is torn down.

    Clean up functions are called in the reverse order
    of registration.
    """
    tglobs = globs(test)
    stack = tglobs.get(key)
    if stack is None:
        stack = tglobs[key] = []
    stack.append((function, args, kw))


def tearDown(test):
    """
    Call all the clean up functions registered for *test*,
    in the reverse of their registration order.
    """
    tglobs = globs(test)
    stack = tglobs.get(key)
    while stack:
        f, p, k = stack.pop()
        f(*p, **k)


def setUpDirectory(test):
    """
    Create and change to a temporary directory, and `register`
    `rmtree` to clean it up. Returns to the starting directory
    when finished.
    """
    tmp = tempfile.mkdtemp()
    register(test, rmtree, tmp)
    here = os.getcwd()
    register(test, os.chdir, here)
    os.chdir(tmp)


def rmtree(path):
    """
    Remove all the files and directories
    found in *path*.

    Intended to be used as a clean up function with *register*.
    """
    for path, dirs, files in os.walk(path, False):
        for fname in files:
            fname = os.path.join(path, fname)
            if not os.path.islink(fname):
                os.chmod(fname, stat.S_IWUSR)
            os.remove(fname)
        for dname in dirs:
            dname = os.path.join(path, dname)
            os.rmdir(dname)
    os.rmdir(path)


def context_manager(test, manager):
    """
    A stack-based version of the **with** statement.

    The context manager *manager* is entered when this
    method is called, and it is exited when the *test*
    is torn down.
    """
    result = manager.__enter__()
    register(test, manager.__exit__, None, None, None)
    return result


def _get_mock():
    # A hook for the setupstack.txt doctest :
    from unittest import mock as mock_module

    return mock_module


def mock(test, *args, **kw):
    """
    Patch an object by calling `unittest.mock.patch`
    with the *args* and *kw* given, and returns the result.

    This will be torn down when the *test* is torn down.
    """
    mock_module = _get_mock()
    return context_manager(test, mock_module.patch(*args, **kw))


class TestCase(unittest.TestCase):
    """
    A ``TestCase`` base class that overrides `tearDown`
    to use the function from this module.

    In addition, it provides other methods using the
    functions in this module:

    - `register`
    - `setUpDirectory`
    - `context_manager`
    - `mock`

    """

    tearDown = tearDown
    register = register
    setUpDirectory = setUpDirectory
    context_manager = context_manager
    mock = mock



##############################################################################
#
# Copyright (c) 2004 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
Support for advanced normalization of output in doctests.

See :doc:`../renormalizing` for documentation.
"""
import doctest


class OutputChecker(doctest.OutputChecker):
    """Pattern-normalizing output checker
    """

    def __init__(self, patterns=None):
        if patterns is None:
            patterns = []
        self.transformers = list(map(self._cook, patterns))

    def __add__(self, other):
        if not isinstance(other, RENormalizing):
            return NotImplemented
        return RENormalizing(self.transformers + other.transformers)

    def _cook(self, pattern):
        if hasattr(pattern, '__call__'):
            return pattern
        regexp, replacement = pattern
        return lambda text: regexp.sub(replacement, text)

    def check_output(self, want, got, optionflags):
        if got == want:
            return True

        for transformer in self.transformers:
            want = transformer(want)
            got = transformer(got)

        if doctest.OutputChecker.check_output(self, want, got, optionflags):
            return True
        return False

    def output_difference(self, example, got, optionflags):
        want = example.want

        # If want is empty, use original outputter. This is useful
        # when setting up tests for the first time.  In that case, we
        # generally use the differencer to display output, which we evaluate
        # by hand.
        if not want.strip():
            return doctest.OutputChecker.output_difference(
                self, example, got, optionflags)

        # Dang, this isn't as easy to override as we might wish
        original = want

        for transformer in self.transformers:
            want = transformer(want)
            got = transformer(got)

        # temporarily hack example with normalized want:
        example.want = want

        result = doctest.OutputChecker.output_difference(
            self, example, got, optionflags)
        example.want = original

        return result


RENormalizing = OutputChecker



import warnings


warnings.warn('zope.testing.testrunner is deprecated in favour of '
              'zope.testrunner', DeprecationWarning, stacklevel=2)

try:
    import zope.testing
    from zope import testrunner

    # Now replace this module with the right one:
    zope.testing.testrunner = testrunner
except ImportError:
    pass



##############################################################################
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""Tests for the testing framework.
"""
import doctest
import re
import sys
import unittest

from zope.testing import renormalizing


def print_(*args):
    sys.stdout.write(' '.join(map(str, args))+'\n')


def setUp(test):
    test.globs['print_'] = print_


def test_suite():
    suite = unittest.TestSuite((
        doctest.DocFileSuite('module.txt'),
        doctest.DocFileSuite('loggingsupport.txt', setUp=setUp),
        doctest.DocFileSuite('renormalizing.txt', setUp=setUp),
        doctest.DocFileSuite('setupstack.txt', setUp=setUp),
        doctest.DocFileSuite('wait.txt', setUp=setUp)
    ))

    suite.addTests(
        doctest.DocFileSuite(
            'doctestcase.txt',
            checker=renormalizing.RENormalizing([
                # for Python 3.11+
                (re.compile(r'\(tests\.MyTest\.test.?\)'), '(tests.MyTest)'),
                (re.compile(r'\(tests.MoreTests.test_.*\)'),
                 '(tests.MoreTests)')
                ])))
    suite.addTests(doctest.DocFileSuite('cleanup.txt'))
    suite.addTests(doctest.DocFileSuite('formparser.txt', setUp=setUp))
    return suite



##############################################################################
#
# Copyright Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################
"""
Provides a class, `Wait`, and an instance, `wait`, that can be
used for polling for a condition to become true, or raise an exception
after a timeout.
"""


import time


class Wait:
    """
    A callable object that polls until *func* returns
    a true value, sleeping *wait* seconds between
    polling attempts.

    If *timeout* seconds elapse, *exception* will be raised.
    """

    class TimeOutWaitingFor(Exception):
        "The default exception raised when a test condition timed out."

    #: The default timeout value.
    timeout = 9
    #: The default amount of time to sleep between polls.
    wait = .01

    def __init__(self,
                 timeout=None, wait=None, exception=None,
                 getnow=lambda: time.time, getsleep=lambda: time.sleep):

        if timeout is not None:
            self.timeout = timeout

        if wait is not None:
            self.wait = wait

        if exception is not None:
            self.TimeOutWaitingFor = exception

        self.getnow = getnow
        self.getsleep = getsleep

    def __call__(self, func=None, timeout=None, wait=None, message=None):
        if func is None:
            return lambda func: self(func, timeout, wait, message)

        if func():
            return

        now = self.getnow()
        sleep = self.getsleep()
        if timeout is None:
            timeout = self.timeout
        if wait is None:
            wait = self.wait
        wait = float(wait)

        deadline = now() + timeout
        while 1:
            sleep(wait)
            if func():
                return
            if now() > deadline:
                raise self.TimeOutWaitingFor(
                    message or
                    getattr(func, '__doc__') or
                    getattr(func, '__name__')
                )


#: The default instance of `Wait`.
wait = Wait()
